#include <GL/glfw.h>
#include "lua.hpp"

int main()
{
    int width;
    int height;
    const char* title;
    int defaultWidth = 512;
    int defaultHeight = 512;
    const char* defaultTitle = "GLFW Not Scripted Application";
    int		frame = 0;
    bool	running = true;
    lua_State* L = lua_open();
    luaL_openlibs(L);

    if(luaL_dofile(L, "config.lua"))
    {
        const char* err = lua_tostring(L, -1);
        width = defaultWidth;
        height = defaultHeight;
        title = defaultTitle;
    }
    else
    {
        lua_getglobal(L, "windowWidth");
        width = (int)lua_tonumber(L, -1);
        if(0 == width)
            width = defaultWidth;

        lua_getglobal(L, "windowHeight");
        height = (int)lua_tonumber(L, -1);
        if(0 == height)
            height = defaultHeight;

        lua_getglobal(L, "windowTitle");
        title = lua_tostring(L, -1);
        if(NULL == title)
            title = defaultTitle;
    }

    glfwInit();

    if( !glfwOpenWindow(width, height, 0, 0, 0, 0, 0, 0, GLFW_WINDOW ) )
    {
        glfwTerminate();
        return 0;
    }

    glfwSetWindowTitle(title);

    while(running)
    {
        frame++;

        glfwGetWindowSize( &width, &height );
        height = height > 0 ? height : 1;

        glViewport( 0, 0, width, height );

        glClearColor( 0.0f, 0.0f, 0.0f, 0.0f );
        glClear( GL_COLOR_BUFFER_BIT );

        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        gluPerspective( 65.0f, (GLfloat)width/(GLfloat)height, 1.0f, 100.0f );

        // Draw some rotating garbage
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();
        gluLookAt(0.0f, -10.0f, 0.0f,
				0.0f, 0.0f, 0.0f,
				0.0f, 0.0f, 1.0f );

        //glTranslatef( 1.0f, 1.0f, 0.0f );
        glRotatef(frame, 0.25f, 1.0f, 0.75f);
        glBegin( GL_TRIANGLES );
          glColor3f(0.1f, 0.0f, 0.0f );
          glVertex3f(0.0f, 3.0f, -4.0f);
          glColor3f(0.0f, 1.0f, 0.0f );
          glVertex3f(3.0f, -2.0f, -4.0f);
          glColor3f(0.0f, 0.0f, 1.0f );
          glVertex3f(-3.0f, -2.0f, -4.0f);
        glEnd();
        glBegin( GL_TRIANGLES );
          glColor3f(0.0f, 0.1f, 0.0f );
          glVertex3f(0.0f, 3.0f, -3.0f);
          glColor3f(0.0f, 0.0f, 1.0f );
          glVertex3f(3.0f, -2.0f, -2.0f);
          glColor3f(1.0f, 0.0f, 0.0f );
          glVertex3f(-3.0f, -2.0f, 2.0f);
        glEnd();
        glfwSwapBuffers();

        // exit if ESC was pressed or window was closed
        running = !glfwGetKey(GLFW_KEY_ESC) && glfwGetWindowParam( GLFW_OPENED);
    }

    glfwTerminate();

    return 0;
}
